use serde::{Deserialize, Serialize};
use thiserror::Error;

use crate::identifier::{Identifier, IdentifierError};

// TODO: Needs to serialize/deserialize as the identifier with a hash in front.
#[derive(Debug, Clone, Serialize, Deserialize)]
#[serde(try_from = "&str", into = "String")]
pub struct Tag(Identifier);

impl TryFrom<&str> for Tag {
    type Error = TagError;
    fn try_from(value: &str) -> Result<Self, Self::Error> {
        match value.strip_prefix("#") {
            Some(v) => Ok(Self(Identifier::try_from(v)?)),
            None => Err(TagError::MissingHash),
        }
    }
}

impl From<Tag> for String {
    fn from(value: Tag) -> Self {
        let v: String = value.0.into();
        format!("#{v}")
    }
}

#[derive(Debug, Error)]
pub enum TagError {
    #[error("Missing hash")]
    MissingHash,
    #[error("Invalid identifier: {0}")]
    InvalidIdentifier(#[from] IdentifierError),
}
