use bytes::{Buf, BufMut};

use crate::packet_encodable::PacketEncodable;

#[derive(Debug)]
pub struct ByteArray(Vec<u8>);

impl PacketEncodable for ByteArray {
    fn encode_packet(
        &self,
        buf: &mut impl BufMut,
    ) -> Result<(), crate::packet_encodable::PacketEncodeError> {
        // Push all data to the buffer, length is not included.
        buf.put_slice(&self.0);

        Ok(())
    }

    fn decode_packet(
        buf: &mut impl Buf,
    ) -> Result<Self, crate::packet_encodable::PacketDecodeError> {
        // Read all remaining data from the cursor. Length should be assumed to be the rest of the
        // message.
        let mut dst = Vec::with_capacity(buf.remaining());
        buf.copy_to_slice(&mut dst);

        Ok(ByteArray(dst))
    }
}
