pub mod clientbound;
pub mod serverbound;

use std::fmt;

use bytes::Bytes;
use tokio::{io::AsyncWriteExt, net::tcp::OwnedWriteHalf};

use crate::{
    datatypes::var_int::VarInt,
    packet_encodable::{PacketDecodeError, PacketEncodable},
};

pub trait Packet: PacketEncodable {
    const HANDSHAKE_ID: i32;
    const STATUS_ID: i32;
    const LOGIN_ID: i32;
    const CONFIGURATION_ID: i32;
    const PLAY_ID: i32;

    fn from_raw(mut raw: RawPacket) -> Result<Self, PacketDecodeError> {
        Self::decode_packet(&mut raw.data)
    }
}

pub struct RawPacket {
    pub id: VarInt,
    pub data: Bytes,
}

impl RawPacket {
    pub async fn write_to_stream(&self, stream: &mut OwnedWriteHalf) -> Result<(), std::io::Error> {
        let length: VarInt = (self.id.encoded_len() + self.data.len()).into();
        length.write_to_stream(stream).await?;
        self.id.write_to_stream(stream).await?;
        stream.write_all(&self.data).await?;
        Ok(())
    }
}

impl fmt::Display for RawPacket {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        let id: i32 = self.id.into();
        write!(f, "{{ id: {}, data: {:X?} }}", id, self.data)
    }
}
