use std::sync::Arc;

use potato_data::{datapack::Datapack, registry::Registries};

pub struct Server {
    pub registries: Arc<Registries>,
}

impl Server {
    #[allow(clippy::new_without_default)]
    pub fn new() -> Server {
        println!("Looking for datapacks...");
        let datapacks: Vec<_> = std::fs::read_dir("datapacks")
            .expect("Failed to read datapacks directory")
            .map(|f| f.unwrap().path())
            .filter(|p| p.is_dir())
            .map(Datapack::new)
            .collect();
        println!("Found {} datapacks", datapacks.len());
        println!("Loading datapacks...");
        let mut registries = Registries::empty();
        datapacks
            .iter()
            .for_each(|pack| pack.load(&mut registries).expect("Failed to load datapack"));

        println!("Finished loading datapacks");
        println!("Loaded {} trim materials", registries.trim_materials.len());
        println!("Loaded {} trim patterns", registries.trim_patterns.len());
        println!("Loaded {} chat types", registries.chat_types.len());
        println!(
            "Loaded {} dimension types",
            registries.dimension_types.len()
        );
        println!(
            "Loaded {} painting variants",
            registries.painting_variants.len()
        );
        println!("Loaded {} wolf variants", registries.wolf_variants.len());
        println!("Loaded {} damage types", registries.damage_types.len());
        println!("Loaded {} biomes", registries.worldgen.biomes.len());

        Server {
            registries: Arc::new(registries),
        }
    }
}
